import numpy as np
import yaml
import matplotlib.pyplot as plt
import xarray as xr


class iFTS_specifications(object):


    def __init__(self, filename = r'iFTS_specifications.yaml', channel = 'O2'):
        """
        This class reads in and assigns the iFTS specifications.
        """


        with open(filename, 'r') as stream:
            try:
                ifts_specs_dict = yaml.full_load(stream)['iFTS Specifications']

            except:
                ifts_specs_dict = yaml.load(stream)['iFTS Specifications']


        with open(filename, 'r') as stream:
            if channel == 'O2':
                try:
                    detector_specs_dict = yaml.full_load(stream)['O2 Detector Specifications']

                except:
                    detector_specs_dict = yaml.load(stream)['O2 Detector Specifications']

            if channel == 'CO2_CH4':
                try:
                    detector_specs_dict = yaml.full_load(stream)['CO2_CH4 Detector Specifications']

                except:
                    detector_specs_dict = yaml.load(stream)['CO2_CH4 Detector Specifications']



        with open(filename, 'r') as stream:

            try:
                entrance_optics_dict = yaml.full_load(stream)['Entrance Optics Specifications']

            except:
                imaging_optics_dict = yaml.load(stream)['Entrance Optics Specifications']

        with open(filename, 'r') as stream:

            try:
                imaging_optics_dict = yaml.full_load(stream)['Aft Optics Specifications']

            except:
                imaging_optics_dict = yaml.load(stream)['Aft Optics Specifications']

        for x in detector_specs_dict:
            detector_specs_dict[x] = float(detector_specs_dict[x])

        # Add all parameters to the design specifications:
        self.design = ifts_specs_dict
        self.design.update(detector_specs_dict)
        self.design.update(imaging_optics_dict)
        self.design.update(entrance_optics_dict)


        self.max_opd = self.design['MaxOPD']
        self.sampling = self.design['Sampling']
        self.modulation_efficiency = self.design['ModulationEfficiency']
        self.total_integration_time = self.design['integration_time']
        self.pupil_diameter = self.design['interferometer_aperture_diameter']
        self.interferometer_fov  = self.design['interferometer_fov']

        self.DetNumPixX = np.int(self.design['DetNumPixX'])
        self.DetNumPixY =  np.int(self.design['DetNumPixY'])

        self.DetWidthPixX = self.design['DetWidthPixX']
        self.DetWidthPixY = self.design['DetWidthPixY']
        self.DetectorQE   = self.design['DetectorQE']
        self.DetectorADU  = self.design['DetectorADU']
        self.DetectorDark = self.design['DetectorDark']
        self.DetectorWell = self.design['DetectorWell']
        self.DetectorReadNoise = self.design['DetectorReadNoise']
        self.DetectorBits      = np.int(self.design['DetectorBits'])
        self.DetectorBias      = self.design['DetectorBias']


        self.ifov        = self.design['ifov']
        self.full_fov    = self.design['interferometer_fov']
        self.Entrance_Pupil = self.design['Entrance_Pupil']
        self.efl           = self.design['efl']
        self.interferometer_fov  = self.design['interferometer_fov']
        self.interferometer_ifov  = self.design['interferometer_ifov']

        self.transmission_beam_1 = self.design['transmission_beam_1']
        self.transmission_beam_2 = self.design['transmission_beam_2']

        self.SolidAngle = np.tan(self.interferometer_ifov*1e-3)**2
        self.pupil_area = np.pi*(self.pupil_diameter/2)**2